pub unsafe trait OwnedProxy: UntypedOwnedProxyWrapper {
type Borrowed: BorrowedProxy<Owned = Self>;
type Api;
type NoOpEventHandler: EventHandler + Send + 'static;
const INTERFACE: &'static str;
const WL_INTERFACE: &'static wl_interface;
const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler;
const MAX_VERSION: u32;
}Expand description
An owned proxy.
This type is usually implemented by bindings that are automatically generated with the
wl-client-builder crate.
Safety
WL_INTERFACEmust refer to a valid interface specification.- It must be safe to transmute this type from an
UntypedOwnedProxythat has an interface that is compatible withWL_INTERFACE. - The interface of the contained proxy must be compatible with
WL_INTERFACE. - The
EventHandler::mutable_typeof theOwnedProxy::NoOpEventHandlermust beNoneor the type ID of().
Required Associated Constants
Sourceconst WL_INTERFACE: &'static wl_interface
const WL_INTERFACE: &'static wl_interface
The libwayland interface specification.
Sourceconst NO_OP_EVENT_HANDLER: Self::NoOpEventHandler
const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler
An event handler that ignores all events without leaking memory.
Sourceconst MAX_VERSION: u32
const MAX_VERSION: u32
The maximum protocol version supported by this type.
Required Associated Types
Sourcetype Borrowed: BorrowedProxy<Owned = Self>
type Borrowed: BorrowedProxy<Owned = Self>
The borrowed version of this proxy.
Sourcetype Api
type Api
A type used to implement some technical operations on the proxy.
This type exists to avoid polluting the function namespace of the proxy.
Sourcetype NoOpEventHandler: EventHandler + Send + 'static
type NoOpEventHandler: EventHandler + Send + 'static
An event handler that ignores all events without leaking memory.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".