Struct WlDisplay

pub struct WlDisplay { /* private fields */ }
An owned wl_display proxy.

See the documentation of the module for the interface description.

Implementations

Source

impl WlDisplay

pub fn on_error<F>(f: F) -> Error<F>

Creates an event handler for error events.

The event handler ignores all other events.

pub fn on_delete_id<F>(f: F) -> DeleteId<F>
where F: Fn(&WlDisplayRef, u32),

Creates an event handler for delete_id events.

The event handler ignores all other events.

Source

impl WlDisplay

pub const REQ__SYNC__SINCE: u32 = 1u32

Since when the sync request is available.

pub const REQ__GET_REGISTRY__SINCE: u32 = 1u32

Since when the get_registry request is available.

pub fn sync(&self) -> WlCallback

pub fn get_registry(&self) -> WlRegistry

Source

impl WlDisplay

pub const EVT__ERROR__SINCE: u32 = 1u32

Since when the error event is available.

pub const EVT__DELETE_ID__SINCE: u32 = 1u32

Since when the delete_id event is available.

Methods from Deref<Target = WlDisplayRef>

pub fn sync(&self, _queue: &Queue) -> WlCallback

Arguments
  • _queue: The queue that the returned proxy is assigned to.
pub fn get_registry(&self, _queue: &Queue) -> WlRegistry

Arguments
  • _queue: The queue that the returned proxy is assigned to.