Struct WlDisplayRef

pub struct WlDisplayRef { /* private fields */ }
A borrowed wl_display proxy.

See the documentation of the module for the interface description.

Implementations

Source

impl WlDisplayRef

pub fn sync(&self, _queue: &Queue) -> WlCallback

Arguments
  • _queue: The queue that the returned proxy is assigned to.
pub fn get_registry(&self, _queue: &Queue) -> WlRegistry

Arguments
  • _queue: The queue that the returned proxy is assigned to.