Struct WlRoot

pub struct WlRoot { /* private fields */ }
An owned wl_root proxy.

See the documentation of the module for the interface description.

Implementations

Source

impl WlRoot

pub fn on_pong_dummy<F>(f: F) -> PongDummy<F>
where F: Fn(&WlRootRef, Option<&WlDummyRef>),

Creates an event handler for pong_dummy events.

The event handler ignores all other events.

pub fn on_new_dummy<F>(f: F) -> NewDummy<F>
where F: Fn(&WlRootRef, WlDummy),

Creates an event handler for new_dummy events.

The event handler ignores all other events.

Source

impl WlRoot

pub const REQ__CREATE_DUMMY__SINCE: u32 = 1u32

Since when the create_dummy request is available.

pub const REQ__DESTROY__SINCE: u32 = 1u32

Since when the destroy request is available.

pub const REQ__GET_SERVER_NAME__SINCE: u32 = 1u32

Since when the get_server_name request is available.

pub const REQ__ECHO__SINCE: u32 = 1u32

Since when the echo request is available.

pub const REQ__BIND__SINCE: u32 = 1u32

Since when the bind request is available.

pub fn create_dummy(&self) -> WlDummy

pub fn destroy(&self)

pub fn get_server_name(&self) -> WlString

pub fn echo(&self, str: &str) -> WlString

Arguments
  • str:
pub fn bind<P: OwnedProxy>(&self, version: u32) -> P

Source

impl WlRoot

pub const EVT__PONG_DUMMY__SINCE: u32 = 1u32

Since when the pong_dummy event is available.

pub const EVT__NEW_DUMMY__SINCE: u32 = 1u32

Since when the new_dummy event is available.

Methods from Deref<Target = WlRootRef>

pub fn create_dummy(&self, _queue: &Queue) -> WlDummy

Arguments
  • _queue: The queue that the returned proxy is assigned to.
pub fn ping_dummy(&self, id: &WlDummyRef)

Arguments
  • id:
pub fn get_server_name(&self, _queue: &Queue) -> WlString

Arguments
  • _queue: The queue that the returned proxy is assigned to.
pub fn send_new_dummy(&self)

pub fn echo(&self, _queue: &Queue, str: &str) -> WlString

Arguments
  • _queue: The queue that the returned proxy is assigned to.
  • str:
pub fn bind<P: OwnedProxy>(&self, _queue: &Queue, version: u32) -> P

Arguments
  • _queue: The queue that the returned proxy is assigned to.