Struct WlRootRef

pub struct WlRootRef { /* private fields */ }
A borrowed wl_root proxy.

See the documentation of the module for the interface description.

Implementations

Source

impl WlRootRef

pub fn create_dummy(&self, _queue: &Queue) -> WlDummy

Arguments
  • _queue: The queue that the returned proxy is assigned to.
pub fn ping_dummy(&self, id: &WlDummyRef)

Arguments
  • id:
pub fn get_server_name(&self, _queue: &Queue) -> WlString

Arguments
  • _queue: The queue that the returned proxy is assigned to.
pub fn send_new_dummy(&self)

pub fn echo(&self, _queue: &Queue, str: &str) -> WlString

Arguments
  • _queue: The queue that the returned proxy is assigned to.
  • str:
pub fn bind<P: OwnedProxy>(&self, _queue: &Queue, version: u32) -> P

Arguments
  • _queue: The queue that the returned proxy is assigned to.