Trait OwnedProxy

pub unsafe trait OwnedProxy: UntypedOwnedProxyWrapper {
    type Borrowed: BorrowedProxy<Owned = Self>;
    type Api;
    type NoOpEventHandler: EventHandler + Send + 'static;

    const INTERFACE: &'static str;
    const WL_INTERFACE: &'static wl_interface;
    const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler;
    const MAX_VERSION: u32;
}
An owned proxy.

This type is usually implemented by bindings that are automatically generated with the wl-client-builder crate.

Safety

  • WL_INTERFACE must refer to a valid interface specification.
  • It must be safe to transmute this type from an UntypedOwnedProxy that has an interface that is compatible with WL_INTERFACE.
  • The interface of the contained proxy must be compatible with WL_INTERFACE.

Required Associated Constants

const INTERFACE: &'static str

The name of the interface.

const WL_INTERFACE: &'static wl_interface

The libwayland interface specification.

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

An event handler that ignores all events without leaking memory.

const MAX_VERSION: u32

The maximum protocol version supported by this type.

Required Associated Types

type Borrowed: BorrowedProxy<Owned = Self>

The borrowed version of this proxy.

type Api

A type used to implement some technical operations on the proxy.

This type exists to avoid polluting the function namespace of the proxy.

type NoOpEventHandler: EventHandler + Send + 'static

An event handler that ignores all events without leaking memory.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors

Source

impl OwnedProxy for WlCallback

Source

const INTERFACE: &'static str = "wl_callback"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 1u32

Source

type Borrowed = WlCallbackRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>

Source

impl OwnedProxy for WlDisplay

Source

const INTERFACE: &'static str = "wl_display"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 1u32

Source

type Borrowed = WlDisplayRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>

Source

impl OwnedProxy for WlDummy

Source

const INTERFACE: &'static str = "wl_dummy"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 1u32

Source

type Borrowed = WlDummyRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>

Source

impl OwnedProxy for WlKeyboard

Source

const INTERFACE: &'static str = "wl_keyboard"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 10u32

Source

type Borrowed = WlKeyboardRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>

Source

impl OwnedProxy for WlRegistry

Source

const INTERFACE: &'static str = "wl_registry"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 1u32

Source

type Borrowed = WlRegistryRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>

Source

impl OwnedProxy for WlRoot

Source

const INTERFACE: &'static str = "wl_root"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 1u32

Source

type Borrowed = WlRootRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>

Source

impl OwnedProxy for WlSeat

Source

const INTERFACE: &'static str = "wl_seat"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 10u32

Source

type Borrowed = WlSeatRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>

Source

impl OwnedProxy for WlString

Source

const INTERFACE: &'static str = "wl_string"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 1u32

Source

type Borrowed = WlStringRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>

Source

impl OwnedProxy for WlSurface

Source

const INTERFACE: &'static str = "wl_surface"

Source

const WL_INTERFACE: &'static wl_interface

Source

const NO_OP_EVENT_HANDLER: Self::NoOpEventHandler

Source

const MAX_VERSION: u32 = 1u32

Source

type Borrowed = WlSurfaceRef

Source

type Api = ProxyApi

Source

type NoOpEventHandler = EventHandler<NoOpEventHandler>