pub struct WlDisplay { /* private fields */ }Expand description
An owned wl_display proxy.
See the documentation of the module for the interface description.
Implementations
Sourceimpl WlDisplay
impl WlDisplay
Sourceimpl WlDisplay
impl WlDisplay
Sourcepub const REQ__SYNC__SINCE: u32 = 1u32
pub const REQ__SYNC__SINCE: u32 = 1u32
Since when the sync request is available.
Sourcepub const REQ__GET_REGISTRY__SINCE: u32 = 1u32
pub const REQ__GET_REGISTRY__SINCE: u32 = 1u32
Since when the get_registry request is available.
pub fn sync(&self) -> WlCallback
pub fn get_registry(&self) -> WlRegistry
Sourceimpl WlDisplay
impl WlDisplay
Sourcepub const EVT__ERROR__SINCE: u32 = 1u32
pub const EVT__ERROR__SINCE: u32 = 1u32
Since when the error event is available.
Sourcepub const EVT__DELETE_ID__SINCE: u32 = 1u32
pub const EVT__DELETE_ID__SINCE: u32 = 1u32
Since when the delete_id event is available.
Methods from Deref<Target = WlDisplayRef>
Sourcepub fn sync(&self, _queue: &Queue) -> WlCallback
pub fn sync(&self, _queue: &Queue) -> WlCallback
Arguments
_queue: The queue that the returned proxy is assigned to.
Sourcepub fn get_registry(&self, _queue: &Queue) -> WlRegistry
pub fn get_registry(&self, _queue: &Queue) -> WlRegistry
Arguments
_queue: The queue that the returned proxy is assigned to.