pub struct WlDisplayRef { /* private fields */ }Expand description
A borrowed wl_display proxy.
See the documentation of the module for the interface description.
Implementations
Sourceimpl WlDisplayRef
impl WlDisplayRef
Sourcepub fn sync(&self, _queue: &Queue) -> WlCallback
pub fn sync(&self, _queue: &Queue) -> WlCallback
Arguments
_queue: The queue that the returned proxy is assigned to.
Sourcepub fn get_registry(&self, _queue: &Queue) -> WlRegistry
pub fn get_registry(&self, _queue: &Queue) -> WlRegistry
Arguments
_queue: The queue that the returned proxy is assigned to.