pub struct WlRoot { /* private fields */ }Expand description
An owned wl_root proxy.
See the documentation of the module for the interface description.
Implementations
Sourceimpl WlRoot
impl WlRoot
Sourcepub fn on_pong_dummy<F>(f: F) -> PongDummy<F>
pub fn on_pong_dummy<F>(f: F) -> PongDummy<F>
Creates an event handler for pong_dummy events.
The event handler ignores all other events.
Sourceimpl WlRoot
impl WlRoot
Sourcepub const REQ__CREATE_DUMMY__SINCE: u32 = 1u32
pub const REQ__CREATE_DUMMY__SINCE: u32 = 1u32
Since when the create_dummy request is available.
Sourcepub const REQ__DESTROY__SINCE: u32 = 1u32
pub const REQ__DESTROY__SINCE: u32 = 1u32
Since when the destroy request is available.
Sourcepub const REQ__GET_SERVER_NAME__SINCE: u32 = 1u32
pub const REQ__GET_SERVER_NAME__SINCE: u32 = 1u32
Since when the get_server_name request is available.
Sourcepub const REQ__ECHO__SINCE: u32 = 1u32
pub const REQ__ECHO__SINCE: u32 = 1u32
Since when the echo request is available.
Sourcepub const REQ__BIND__SINCE: u32 = 1u32
pub const REQ__BIND__SINCE: u32 = 1u32
Since when the bind request is available.
pub fn create_dummy(&self) -> WlDummy
pub fn destroy(&self)
pub fn get_server_name(&self) -> WlString
pub fn bind<P: OwnedProxy>(&self, version: u32) -> P
Sourceimpl WlRoot
impl WlRoot
Sourcepub const EVT__PONG_DUMMY__SINCE: u32 = 1u32
pub const EVT__PONG_DUMMY__SINCE: u32 = 1u32
Since when the pong_dummy event is available.
Sourcepub const EVT__NEW_DUMMY__SINCE: u32 = 1u32
pub const EVT__NEW_DUMMY__SINCE: u32 = 1u32
Since when the new_dummy event is available.
Methods from Deref<Target = WlRootRef>
Sourcepub fn create_dummy(&self, _queue: &Queue) -> WlDummy
pub fn create_dummy(&self, _queue: &Queue) -> WlDummy
Arguments
_queue: The queue that the returned proxy is assigned to.
Sourcepub fn ping_dummy(&self, id: &WlDummyRef)
pub fn ping_dummy(&self, id: &WlDummyRef)
Arguments
id:
Sourcepub fn get_server_name(&self, _queue: &Queue) -> WlString
pub fn get_server_name(&self, _queue: &Queue) -> WlString
Arguments
_queue: The queue that the returned proxy is assigned to.
pub fn send_new_dummy(&self)