pub struct WlRootRef { /* private fields */ }Expand description
A borrowed wl_root proxy.
See the documentation of the module for the interface description.
Implementations
Sourceimpl WlRootRef
impl WlRootRef
Sourcepub fn create_dummy(&self, _queue: &Queue) -> WlDummy
pub fn create_dummy(&self, _queue: &Queue) -> WlDummy
Arguments
_queue: The queue that the returned proxy is assigned to.
Sourcepub fn ping_dummy(&self, id: &WlDummyRef)
pub fn ping_dummy(&self, id: &WlDummyRef)
Arguments
id:
Sourcepub fn get_server_name(&self, _queue: &Queue) -> WlString
pub fn get_server_name(&self, _queue: &Queue) -> WlString
Arguments
_queue: The queue that the returned proxy is assigned to.
pub fn send_new_dummy(&self)